/*
 * This file is part of FGGL.
 *
 * FGGL is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any
 * later version.
 *
 * FGGL is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License along with FGGL.
 * If not, see <https://www.gnu.org/licenses/>.
 */

//
// Created by webpigeon on 23/07/22.
//

#include <map>
#include <string>
#include <cassert>

#include "fggl/util/guid.hpp"

namespace fggl::util {
	namespace {
		std::map<GUID, std::string> guidTable;
	}

	GUID internString(const char* str) {
		assert(str != nullptr);

		GUID guid = make_guid(str);
		auto tableValue = guidTable.find(guid);
		if (tableValue != guidTable.end()) {
			assert(tableValue->second == str);
		} else {
			guidTable[guid] = str;
		}
		return guid;
	}

	std::string guidToString(GUID guid) {
		auto tableValue = guidTable.find(guid);
		if (tableValue != guidTable.end()) {
			return tableValue->second;
		}
		// it's not in the table...
		return "UNKNOWN_GUID("+std::to_string(guid.get())+")";
	}

}

fggl::util::GUID operator "" _fid(const char* str) {
	fggl::util::internString(str);
	return fggl::util::make_guid(str);
}

fggl::util::GUID operator "" _fid(const char* str, std::size_t) {
	fggl::util::internString(str);
	return fggl::util::make_guid(str);
}
