/*
 * This file is part of FGGL.
 *
 * FGGL is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any
 * later version.
 *
 * FGGL is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License along with FGGL.
 * If not, see <https://www.gnu.org/licenses/>.
 */

//
// Created by webpigeon on 27/06/22.
//

#ifndef FGGL_DISPLAY_GLFW_MODULE_HPP
#define FGGL_DISPLAY_GLFW_MODULE_HPP

#include "fggl/display/window.hpp"
#include "fggl/input/module.hpp"

#include "fggl/display/glfw/window.hpp"
#include "fggl/display/glfw/services.hpp"

namespace fggl::display {

	struct GLFW {
		constexpr static const char* name = "fggl::display::glfw";
		constexpr static const std::array<modules::ModuleService, 1> provides = {
			WindowService::service
		};
		constexpr static const std::array<modules::ModuleService, 2> depends = {
			fggl::input::Input::service,
			fggl::gfx::WindowGraphics::service
		};

		static bool factory(modules::ModuleService name, modules::Services& serviceManager);
	};

	bool GLFW::factory(modules::ModuleService service, modules::Services& services) {
		if (service == WindowService::service) {
			auto input = services.get<input::Input>();
			auto graphics = services.get<gfx::WindowGraphics>();

			auto context = std::make_shared<glfw::GlfwContext>(input);
			services.bind<WindowService, glfw::WindowService>(context, graphics);
			return true;
		}
		return false;
	}

} // namespace fggl::display

#endif //FGGL_DISPLAY_GLFW_MODULE_HPP
