/*
 * This file is part of FGGL.
 *
 * FGGL is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any
 * later version.
 *
 * FGGL is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License along with FGGL.
 * If not, see <https://www.gnu.org/licenses/>.
 */

//
// Created by webpigeon on 27/06/22.
//

#ifndef FGGL_DISPLAY_GLFW_SERVICES_HPP
#define FGGL_DISPLAY_GLFW_SERVICES_HPP

#include "fggl/display/window.hpp"
#include "fggl/gfx/setup.hpp"
#include <memory>
#include <utility>
#include <vector>

namespace fggl::display::glfw {

	class WindowService : public display::WindowService {
		public:
			explicit WindowService(std::shared_ptr<GlfwContext> context, gfx::WindowGraphics* gfx) : m_context(std::move(context)), m_gfx(gfx), m_windows() {}
			virtual ~WindowService() = default;

			display::Window* create() override {
				m_windows.push_back(std::make_unique<Window>(m_context, m_gfx));
				return m_windows.back().get();
			}

			void pollEvents() override {
				m_context->pollEvents();
			}

		private:
			std::shared_ptr<GlfwContext> m_context;
			gfx::WindowGraphics* m_gfx;
			std::vector<std::unique_ptr<Window>> m_windows;
	};

} // namespace fggl::display

#endif //FGGL_DISPLAY_GLFW_SERVICES_HPP
