/*
 * This file is part of FGGL.
 *
 * FGGL is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any
 * later version.
 *
 * FGGL is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License along with FGGL.
 * If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef FGGL_GUI_CONTAINERS_HPP
#define FGGL_GUI_CONTAINERS_HPP

#include <fggl/gui/widget.hpp>

namespace fggl::gui {

	class Container : public Widget {
		public:
			Container() = default;
			virtual ~Container() = default;

			inline void clear() {
				m_children.clear();
			}

			void add(std::unique_ptr<Widget> widget);
			virtual inline void layout() {}

			bool contains(const math::vec2 &point) override;
			Widget *getChildAt(const math::vec2 &point) override;
			void render(gfx::Paint &paint) override;

		private:
			bool m_dirty;

		protected:
			std::vector<std::unique_ptr<Widget>> m_children;
	};

	class Panel : public Container {
		public:
			Panel() = default;
			~Panel() = default;

			void render(gfx::Paint& paint) override;

	};

	enum class LayoutAxis {
		LINE_AXIS,
		PAGE_AXIS
	};

	/**
	 * A box-like container.
	 * This container lines child elements up along an axis.
	 *
	 * This is a fairly common approach in may layout frameworks, for example:
	 *   * Java's BoxLayout
	 *   * GTK's Boxes
	 *   * QT's QBoxLayout
	 *   * Android's XML box things
	 */
	/*class Box : public Container {
		public:
			Box(LayoutAxis axis);
			void layout() override;
		private:
			const LayoutAxis m_axis;
	};*/

}; //namespace fggl::gui

#endif
