/*
 * This file is part of FGGL.
 *
 * FGGL is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any
 * later version.
 *
 * FGGL is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License along with FGGL.
 * If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef FGGL_GUI_WIDGETS_HPP
#define FGGL_GUI_WIDGETS_HPP

#include <functional>
#include <utility>

#include "fggl/gui/widget.hpp"
#include "fggl/gui/fonts.hpp"

namespace fggl::gui {

	using Callback = std::function<void(void)>;

	class Label : public Widget {
		public:
			Label() = default;
			Label(math::vec2 pos, math::vec2 size);

			~Label() = default;

			void render(gfx::Paint &paint) override {
				auto pos = topLeft();
				auto size2 = size();
				math::vec2 baseLine{pos.x + 10, pos.y + size2.y/2 + 5};
				paint.text(m_value,baseLine);
			}

			inline void font(std::shared_ptr<FontFace> font) {
				m_font = std::move(font);
				m_needsLayout = true;
			}

			inline void text(const std::string& value) {
				m_value = value;
				m_needsLayout = true;
			}

			inline std::string text() const {
				return m_value;
			}

			math::vec2 naturalSize() {
				if ( m_needsLayout ) {
					layout();
				}
				return m_naturalSize;
			}

			void layout();

		private:
			std::shared_ptr<gui::FontFace> m_font;
			std::string m_value;
			bool m_needsLayout;
			math::vec2 m_naturalSize;
	};

	class Button : public Widget {
		public:
			Button() = default;
			Button(math::vec2 pos, math::vec2 size);

			void render(gfx::Paint &paint) override;

			void activate() override;
			void onEnter() override;
			void onExit() override;

			void label(const std::string& value);

			[[nodiscard]]
			std::string label() const;

			void addCallback(Callback callback);
		private:
			Label m_label;
			std::string m_value;
			std::vector<Callback> m_callbacks;
			bool m_hover;
			bool m_active;
	};


	class Toggle : public Button {
		public:
			Toggle() = default;
	};

}; //namespace fggl::gui

#endif
