/*
 * This file is part of FGGL.
 *
 * FGGL is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any
 * later version.
 *
 * FGGL is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License along with FGGL.
 * If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef FGGL_MATH_FMATH_HPP
#define FGGL_MATH_FMATH_HPP

#include <cmath>

namespace fggl::math {

	// wrap value in range [0, max)
	inline float wrap(float value, float max) {
		return fmod(max + fmod(value, max), max);
	}

	// wrap value in range [min, max)
	inline float wrap(float value, float min, float max) {
		if ( min > max ){ std::swap(min, max); };
		value -= min;
		float rangeSize = max - min;
		return value - (rangeSize* std::floor(value/rangeSize)) + min;
	}

	// if the value is out of bounds, return that bound
	inline float clamp(float value, float min, float max) {
		const float t = value < min ? min : value;
		return t > max ? max : t;
	}

} // namespace fggl::math

#endif //FGGL_MATH_FMATH_HPP
