/*
 * This file is part of FGGL.
 *
 * FGGL is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any
 * later version.
 *
 * FGGL is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License along with FGGL.
 * If not, see <https://www.gnu.org/licenses/>.
 */

//
// Created by webpigeon on 20/11/2021.
//

#ifndef FGGL_SCENES_SCENE_HPP
#define FGGL_SCENES_SCENE_HPP

#include <memory>
#include <string>
#include <unordered_map>

namespace fggl::scenes {

	class Scene {
		public:
			virtual ~Scene() = default;

			virtual void setup() = 0;
			virtual void cleanup() = 0;

			virtual void update() = 0;
			virtual void render() = 0;
	};

	class SceneManager {
		public:
			SceneManager() = default;

			void create(const std::string &name, std::shared_ptr<Scene> scene);
			void activate(const std::string &name);

			inline void update() {
				if (m_active == nullptr) {
					return;
				}
				m_active->update();
			}

			inline void render() {
				if (m_active == nullptr) {
					return;
				}
				m_active->render();
			}

		private:
			std::shared_ptr<Scene> m_active;
			std::unordered_map<std::string, std::shared_ptr<Scene>> m_scenes;
	};

}

#endif //FGGL_SCENES_SCENE_HPP
